math.randomseed(os.time())

import "particles"
import "level"
import "audio"

if first_time == nil then
    first_time = true
end

local images = lt.LoadImages({
    "footnote", "title",
    "paint1",
    "paint2",
    "paint3",
    "paint4",
    "paint5",
    "paint6",
    "paint7",
    "paint8",
    "paint9",
    "paint10",
    "platform",
    "platform2",
    "run1",
    "run2",
    "run3",
    "stand",
    "bulb",
    "dollar",
    "heart",
    "bucket",
    "fan",
    "knife",
    "pen",
    "brush",
    "fan",
    "rects",
    "bg",
    "line",
    "dropper",
    {font = "font", glyphs = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"},
    }, "nearest", "nearest")

images.platform = images.platform:Scale(2):Translate(0, -images.platform.height)
images.platform2 = images.platform2:Scale(2):Translate(0, images.platform2.height - 8)

lt.FixGlobals()

local
function start()

    local gravity = 17
    local run_speed = 290
    local run_force = 0.5
    local jump_impulse = 320
    local start_jump_force = 0.57
    local jump_force_decay = 1.4
    local max_jump_time = 1
    local prejump_time = 0.1
    local brush_lifetime = 5
    local bar_width = 300
    local bar_height = 18

    local backing_color = {1, 1, 1}

    local main_layer = lt.Layer()
    local world = box2d.World(0, -gravity)
    world.auto_clear_forces = true
    --world.debug = true
    world.scale = 100

    local inc_indicator
    local get_indicator

    local indicator_colors = {
        health = {0.7, 0.4, 0.3},
        inspiration = {0.4, 0.8, 0.9},
        money = {0.6, 0.6, 0.2},
        brush = {1, 1, 1},
        fan = {1, 1, 1},
        rects = {1, 1, 1},
        lines = {1, 1, 1},
        color = {1, 1, 1},
        scrape = {1, 1, 1},
        bg = {1, 1, 1},
        pen = {1, 1, 1},
    }

    local indicator_image = {
        health = images.heart,
        inspiration = images.bulb,
        money = images.dollar,
        brush = images.brush,
        fan = images.fan,
        rects = images.rects,
        lines = images.line,
        color = images.dropper,
        scrape = images.knife,
        bg = images.bg,
        pen = images.pen,
    }

    local
    function make_bar(label, val, color)
        local bar = lt.Layer()
        local bg = lt.Rect(0, 0, bar_width, bar_height):Tint(0.5, 0.5, 0.5)
        local fg = lt.Rect(0, 0, bar_width * val, bar_height):Tint(unpack(color))
        bar:Insert(bg:Translate(2, -2):Tint(0, 0, 0))
        bar:Insert(bg)
        bar:Insert(fg)
        bar:Insert(indicator_image[label]:Translate(-18, 8):Tint(0, 0, 0))
        bar:Insert(indicator_image[label]:Translate(-20, 10):Tint(unpack(color)))
        bar.set_val = function(v)
            fg.x2 = v * bar_width
        end
        local alt
        bar:Action(function(dt)
            local limit = 0.2
            if fg.x2 < (bar_width) * limit and alt then
                bg.red = 1
                bg.green = 1
                bg.blue = 1
            elseif fg.x2 < (bar_width) * limit then
                bg.red = 0
                bg.green = 0
                bg.blue = 0
            else
                bg.red = 0.5
                bg.green = 0.5
                bg.blue = 0.5
            end
            alt = not alt
            return 0.1
        end)
        return bar
    end

    local
    function add_indicators() 
        local vals = {
            health = 1,
            inspiration = 1,
            money = 1,
        }

        local indicators = lt.Layer()
        local health_bar = make_bar("health", vals.health, indicator_colors.health)
        local inspiration_bar = make_bar("inspiration", vals.inspiration, indicator_colors.inspiration)
        local money_bar = make_bar("money", vals.money, indicator_colors.money)
        indicators:Insert(inspiration_bar)
        indicators:Insert(money_bar:Translate(0, bar_height * 1.6 * 1))
        indicators:Insert(health_bar:Translate(0, bar_height * 1.6 * 2))
        main_layer:Insert(indicators:Translate(170, 240))

        local bar_map = {
            health = health_bar,
            inspiration = inspiration_bar,
            money = money_bar,
        }

        local alt = false
        indicators:Action(function(dt)
            vals.health = vals.health - dt * 0.07 * math.random()
            health_bar.set_val(vals.health)
            vals.inspiration = vals.inspiration - dt * 0.07 * math.random()
            inspiration_bar.set_val(vals.inspiration)
            vals.money = vals.money - dt * 0.07 * math.random()
            money_bar.set_val(vals.money)
        end)

        function get_indicator(indicator)
            return vals[indicator]
        end

        function inc_indicator(indicator, amt)
            vals[indicator] = math.min(1, vals[indicator] + amt)
            bar_map[indicator].set_val(vals[indicator])
        end
    end

    local platforms = {}
    local platforms_layer = lt.Layer()

    local
    function new_platforms()
        local platforms_new = {}
        local platforms_old = table.clone(platforms)
        for i, platform in ipairs(platforms) do
            if platform.has_brush then
                table.insert(platforms_new, platform)
                table.remove(platforms_old, i)
            end
        end

        local platforms_old2 = table.clone(platforms_old)
        platforms = platforms_new

        local buf = 20
        if first_time then
            platforms = generate_platforms(platforms, lt.left + buf, lt.bottom + buf / 2, lt.right - buf, 0)
        else
            platforms = generate_platforms(platforms, lt.left + buf, lt.bottom + buf / 2, lt.right - buf, lt.top - buf * 4)
        end

        -- match up old and new platforms for transition animations
        local animation_pairs = {}
        for _, platform in ipairs(platforms) do
            if not platform.has_brush then
                -- find closest old platform of same type
                local d = 10000000
                local closest
                for i, platform_old in ipairs(platforms_old) do
                    if platform_old.type == platform.type then
                        local d1 = math.sqrt((platform_old.x1 - platform.x1) ^ 2 + (platform_old.y1 - platform.y1) ^ 2)
                        if d1 < d then
                            closest = i
                            d = d1
                        end
                    end
                end
                if closest then
                    animation_pairs[platform] = platforms_old[closest]
                    table.remove(platforms_old, closest)
                end
            end
        end

        for _, p in ipairs(platforms) do
            if not p.has_brush then
                local w, h = p.x2 - p.x1, p.y2 - p.y1
                local partner = animation_pairs[p]
                local plat_body = world:Body({type = "static"})
                plat_body:Polygon({p.x1, p.y1, p.x2, p.y1, p.x2, p.y2, p.x1, p.y2},
                    {restitution = 0, friction = 0}).platform = true
                if partner then
                    plat_body.child = partner.body.child
                else
                    local img = images[p.img_name]
                    if p.img_name == "platform" then
                        plat_body.child = img
                            :Translate(p.x1 + w/2, lt.bottom):Tint(1, 1, 1)
                    else
                        plat_body.child = img
                            :Translate(p.x1 + w/2, lt.top):Tint(1, 1, 1)
                    end
                end
                plat_body.child:Tween{y = p.y2, x = p.x1 + w/2, time = 0.3}
                platforms_layer:Insert(plat_body)
                p.body = plat_body
            end
        end

        for _, platform in ipairs(platforms_old2) do
            platforms_layer:Remove(platform.body)
            platform.body:Destroy()
        end
    end

    local
    function remove_platforms()
        for _, platform in ipairs(platforms) do
            platform.body.child:Tween{alpha = 0, time = 0.2, action = function()
                platforms_layer:Remove(platform.body)
                platform.body:Destroy()
            end}
        end
        platforms = {}
    end

    new_platforms()

    local player_start_platform = platforms[1]
    local player_body = world:Body({type = "dynamic", fixed_rotation = true, damping = 0.8})
    local player_fixture = player_body:Polygon({-5, 20, 5, 20, 5, 0, -5, 0}, {restitution = 0, friction = 0})
    player_body.x = (player_start_platform.x2 - player_start_platform.x1) / 2 + player_start_platform.x1
    player_body.y = player_start_platform.y2 + 30

    local frame = 1
    local player_frame = images.run1:Scale(1)
        :Action(function(dt, node)
            frame = frame + 1
            if frame > 3 then
                frame = 1
            end
            if math.abs(player_body.vx) > 0.5 then
                node.child = images["run" .. frame]
            else
                node.child = images.stand
            end
            return 0.1
        end)
        :Action(function(dt, node)
            if player_body.vx > 0 then
                node.scale_x = 1
            else
                node.scale_x = -1
            end
        end)

    local player_node = 
        lt.Layer(
            player_frame,
            player_frame:Tint(0, 0, 0, 1):Translate(0.5, 0.5))
            :Scale(4)
            :Translate(0, images.run1.height*2)

    player_body.child = player_node

    local run_dir = nil
    local is_jumping = false
    local jump_time = 0
    local prejump_timer = 0

    local jump_force = 0

    local
    function jump()
        player_body.vy = jump_impulse
        is_jumping = true
        jump_force = start_jump_force
    end

    local trail = trail_particles(images["paint" .. math.random(10)], 0)
    trail.emission_rate = 0
    trail.source_position_x = player_body.x
    trail.source_position_y = player_body.y

    local keys_down = 0

    main_layer:KeyDown(function(event)
        if event.key == "left" or event.key == "A" then
            run_dir = "left"
        elseif event.key == "right" or event.key == "D" then
            run_dir = "right"
        elseif event.key == "up" or event.key == "W" then
            if not is_jumping then
                prejump_timer = prejump_time
            end
        elseif event.key == "R" then
            start()
        elseif event.key == "esc" then
            lt.Quit()
        end
        keys_down = keys_down + 1
    end)
    main_layer:KeyUp(function(event)
        if
            (event.key == "left" or event.key == "A") and run_dir == "left" or
            (event.key == "right" or event.key == "D") and run_dir == "right"
        then
            run_dir = nil
        elseif event.key == "up" or event.key == "W" then
            is_jumping = false
            prejump_timer = 0
            jump_time = 0
        end
        keys_down = keys_down - 1
    end)

    --[[
    main_layer:Action(function(dt)
        local gamepad = lt.ReadGamePadState(1, "present")
        if gamepad and keys_down == 0 then
            local x = lt.ReadGamePadState(1, "lstick_x")
            if x < -0.4 then
                run_dir = "left"
            elseif x > 0.4 then
                run_dir = "right"
            else
                run_dir = nil
            end
            local A = lt.ReadGamePadState(1, "A")
            if A then
                if not is_jumping then
                    prejump_timer = prejump_time
                end
            else
                if is_jumping then
                    is_jumping = false
                    prejump_timer = 0
                    jump_time = 0
                end
            end
            if lt.ReadGamePadState(1, "B") then
                import "main"
            end
        end
    end)
    ]]

    local aspect = (lt.right - lt.left) / (lt.top - lt.bottom)
    local canvas = lt.RenderTarget(nil, aspect * 260, 260,
        lt.left, lt.bottom, lt.right, lt.top,
        lt.left, lt.bottom, lt.right, lt.top,
        "nearest", "nearest")
    lt.Layer():Draw(canvas, 0, 0, 0, 0)
    local canvas_blend = canvas:BlendMode("normal")
    --local backing = lt.RenderTarget(nil, aspect * 260, 260,
    --    lt.left, lt.bottom, lt.right, lt.top,
    --    lt.left, lt.bottom, lt.right, lt.top,
    --    "nearest", "nearest")
    local bg_rect = 
        lt.Rect(lt.left, lt.bottom, lt.right, lt.top):Tint(backing_color[1], backing_color[2], backing_color[3])
    --bg_rect:Draw(backing)
    --bg_rect:Draw(canvas)
    if first_time then
        images.title:Scale(3.2):Translate(lt.left + 320, lt.top - 160)
            :Tint(0, 0, 0)
            :Draw(canvas)
    end

    local music = make_music()
    local curr_action

    main_layer:Insert(bg_rect)
    --main_layer:Insert(backing)
    main_layer:Insert(canvas_blend)
    main_layer:Insert(player_body)
    main_layer:Insert(platforms_layer)
    add_indicators()
    main_layer:Insert(trail:Hidden())
    main_layer:Insert(music)
    main_layer:Action(function(dt)
        if curr_action then
            local r = curr_action(dt)
            if r ~= true then
                return r
            end
        end
    end)

    local brush_bodies = {}
    local game_over = false

    local
    function remove_brushes()
        for _, brush_body in ipairs(brush_bodies) do
            platforms_layer:Remove(brush_body)
            brush_body:Destroy()
        end
        brush_bodies = {}
    end

    local
    function end_game()
        curr_action = nil
        music.fadeout()
        remove_platforms()
        main_layer:Remove(player_body)
        player_body:Destroy()
        player_body = nil
        trail.emission_rate = 0
        main_layer:Remove(canvas)
        local border1 = 4
        local border2 = 40
        local border3 = 50
        local border4 = 300
        local d = 1
        local frame = lt.Layer(
            images.footnote:Tint(0, 0, 0):Translate(0, lt.bottom - border3 - 27),
            lt.Rect(lt.left - border1, lt.bottom - border1, lt.left + d, lt.top - d):Tint(0.1, 0.1, 0.1),
            lt.Rect(lt.left - border1, lt.bottom - border1, lt.right + border1, lt.bottom + d):Tint(0.1, 0.1, 0.1),
            lt.Rect(lt.right - d, lt.bottom - border1, lt.right + border1, lt.top + border1):Tint(0.3, 0.3, 0.3),
            lt.Rect(lt.left - border1, lt.top - d, lt.right - d, lt.top + border1):Tint(0.6, 0.6, 0.6),
            canvas_blend,
            --backing,
            bg_rect,
            lt.Rect(lt.left - border2, lt.bottom - border2, lt.right + border2, lt.top + border2):Tint(0.8, 0.8, 0.8),
            lt.Rect(lt.left - border3, lt.bottom - border3, lt.right + border3, lt.top + border3):Tint(0.0, 0.0, 0.0),
            lt.Rect(lt.left - border4, lt.bottom - border4, lt.right + border4, lt.top + border4):Tint(0.4, 0.4, 0.4)
        ):Scale(1):Translate(0)
        main_layer:Insert(frame)
        main_layer:Insert(platforms_layer)
        frame:Tween{scale = 0.75, y = 10, time = 1}
        remove_brushes()
        game_over = true
    end

    local brushes = 0
    local brush_period = 4
    local special_brush_period = 8

    local
    function add_brush(indicator, action)
        local color = indicator_colors[indicator]
        local img = indicator_image[indicator]
        local brush_platform
        local tries = 0
        repeat
            brush_platform = platforms[math.random(#platforms - 1) + 1]
            tries = tries + 1
        until tries == 100 or not brush_platform.has_brush
        brush_platform.has_brush = true
        local brush_body = world:Body({type = "static"})
        table.insert(brush_bodies, brush_body)
        local brush_fixture = brush_body:Polygon({-5, 20, 5, 20, 5, 0, -5, 0}, {sensor = true})
        brush_fixture.brush = true
        brush_fixture.action = action
        brush_fixture.brush_platform = brush_platform
        brush_body.x = (brush_platform.x2 - brush_platform.x1) / 2 + brush_platform.x1
        brush_body.y = brush_platform.y2

        local bucket = img:Translate(0, img.height/2):Scale(2):Tint(0, 0, 0)
        brush_body.child = bucket
        local brush_timer = (first_time and brush_lifetime * 2) or brush_lifetime
        local white = false
        brush_body
        :Action(function(dt)
            if white then
                bucket.red = 0
                bucket.green = 0
                bucket.blue = 0
            else
                bucket.red = color[1]
                bucket.green = color[2]
                bucket.blue = color[3]
            end
            white = not white
            return 0.1
        end)
        platforms_layer:Insert(brush_body)
    end


    local brush_color = {0, 0, 0, 0}

    local
    function new_color()
        local old_color = brush_color
        repeat 
            brush_color = {math.random(), math.random(), math.random(), math.random() * 0.3 + 0.7}
        until math.abs(old_color[1] - brush_color[1]) +
            math.abs(old_color[2] - brush_color[2]) +
            math.abs(old_color[3] - brush_color[3]) > 1.4
        trail.start_color_red = brush_color[1]
        trail.start_color_green = brush_color[2]
        trail.start_color_blue = brush_color[3]
        trail.start_color_alpha = brush_color[4]
        trail.start_color_variance_red = 0.3 * math.random()
        trail.start_color_variance_green = 0.3 * math.random()
        trail.start_color_variance_blue = 0.3 * math.random()
        trail.end_color_red = trail.start_color_red
        trail.end_color_green = trail.start_color_green
        trail.end_color_blue = trail.start_color_blue
        trail.end_color_alpha = trail.start_color_alpha
        trail.end_color_variance_red = 0.3 * math.random()
        trail.end_color_variance_green = 0.3 * math.random()
        trail.end_color_variance_blue = 0.3 * math.random()
    end
    new_color()

    local
    function new_brush()
        trail.img = images["paint" .. math.random(10)]
        trail.emission_rate = math.random() * 100 + 150
        trail.gravity_y = -200 --math.random() * 100 - 50
        trail.gravity_x = 0 --math.random() * 100 - 50
        trail.speed = 2 + ((math.random(2) == 2 and math.random() * 60) or 0)
        trail.angle = math.random() * 360
        trail.angle_variance = math.random() * 360
        trail.source_position_variance_y = math.random() * 20
        trail.source_position_variance_x = math.random() * 20
        trail.start_size = 0.1 + math.random() * 4
        trail.start_size_variance = 0.1 + math.random() * 4
        trail.end_size = 0.1 + math.random() * 4
        trail.end_size_variance = 0.1 + math.random() * 4
        trail.mode = "normal"
        curr_action = nil
    end

    local
    function new_pen()
        trail.img = images["paint" .. math.random(10)]
        trail.emission_rate = math.random() * 200 + 350
        trail.gravity_y = 0
        trail.gravity_x = 0
        trail.speed = math.random() * 0.1
        trail.angle = 0
        trail.angle_variance = 0
        trail.source_position_variance_y = 0
        trail.source_position_variance_x = 0
        trail.start_size = 0.4 + math.random() * 3
        trail.start_size_variance = 0
        trail.end_size = trail.start_size + math.random() * 1.5
        trail.end_size_variance = 0
        trail.mode = "normal"
        curr_action = nil
    end

    local
    function scrape() 
        local r
        local canvas2 = lt.RenderTarget(nil, aspect * 260, 260,
            lt.left*math.random(), lt.bottom*math.random(), lt.right*math.random(), lt.top*math.random(),
            lt.left*math.random(), lt.bottom*math.random(), lt.right*math.random(), lt.top*math.random(),
            "nearest", "nearest")
        canvas:Draw(canvas2)
        canvas2 = canvas2:BlendMode("normal"):Translate(0, 0):Rotate(0):Scale(1)
        local x, y, scale_x, scale_y, angle = 0, 0, 1, 1, 0
        x = math.random() * 300 - 150
        y = math.random() * 300 - 150
        scale_x = math.random() * 2 + 0.3
        scale_y = math.random() * 2 + 0.3
        angle = math.random() * 90 - 45
        r = math.random(3)
        if r == 1 then
            canvas2.mode = "add"
        elseif r == 2 then
            canvas2.mode = "subtract"
        end
        canvas:Action(coroutine.wrap(function(dt)
            for i = 1, 50 do
                local p = i/50
                local a = (canvas2.mode == "add" or canvas2.mode == "subtract") and (0.1 + math.random() * 0.2) or 1
                canvas2:Tint(1, 1, 1, a):Draw(canvas)
                canvas2.x = p * x
                canvas2.y = p * y
                canvas2.scale_x = 1 + p * (scale_x - 1)
                canvas2.scale_y = 1 + p * (scale_y - 1)
                canvas2.angle = p * angle
                coroutine.yield()
            end
            return true
        end))
    end

    local
    function new_bg()
        bg_rect:Tween{red = math.random(), green = math.random(), blue = math.random(), time = 0.4}
    end

    local
    function rects()
        trail.emission_rate = 0
        curr_action = coroutine.wrap(function(dt)
            while true do
                if player_body then
                    local x1, y1 = player_body.x, player_body.y
                    coroutine.yield(math.random() * 0.2)
                    if player_body then
                        local x2, y2 = player_body.x, player_body.y
                        lt.Rect(x1, y1, x2, y2)
                            :Tint(brush_color[1], brush_color[2], brush_color[3], 1):Draw(canvas)
                    end
                end
            end
        end)
    end

    local
    function lines()
        trail.emission_rate = 0
        curr_action = coroutine.wrap(function(dt)
            local thickness = math.random() * 8 + 1
            while true do
                if player_body then
                    local x1, y1 = player_body.x, player_body.y
                    coroutine.yield(math.random() * 0.6)
                    if player_body then
                        local x2, y2 = player_body.x, player_body.y
                        local midx = (x2 - x1) / 2 + x1
                        local midy = (y2 - y1) / 2 + y1
                        local angle = math.atan2(y2 - y1, x2 - x1)
                        local length = math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2)
                        lt.Rect(-length/2, -thickness, length/2, thickness):Rotate(angle * lt.degrees_per_radian)
                            :Translate(midx, midy)
                            :Tint(brush_color[1], brush_color[2], brush_color[3], 1):Draw(canvas)
                    end
                end
            end
        end)
    end

    local
    function fan()
        trail.emission_rate = 500
        trail.gravity_x = math.random() * 400 - 200
        trail.gravity_y = math.random() * 400 - 200
        curr_action = function(dt)
            trail.gravity_y = trail.gravity_y + math.random() * 200 - 100
            trail.gravity_x = trail.gravity_x + math.random() * 200 - 100
            return math.random()
        end
    end

    local got_brush = false
    local
    function make_new_collectables()
        brushes = brushes + 1

        for _, brush_body in ipairs(brush_bodies) do
            platforms_layer:Remove(brush_body)
            brush_body:Destroy()
        end
        brush_bodies = {}

        local is_brush = false
        if math.random(2) == 1 or get_indicator("health") < 0.5 then
            add_brush("health", function()
                inc_indicator("health", 0.3)
            end)
            is_brush = true
        end
        if math.random(2) == 1 or get_indicator("inspiration") < 0.5 then
            add_brush("inspiration", function()
                inc_indicator("inspiration", 0.3)
            end)
            is_brush = true
        end
        if math.random(2) == 1 or get_indicator("money") < 0.5 or not is_brush then
            add_brush("money", function()
                inc_indicator("money", 0.3)
            end)
        end

        local r = math.random() * (got_brush and 110 or 40)
        if r < 18 then
            add_brush("brush", function()
                new_brush()  
                if not got_brush then
                    music.change()
                    got_brush = true
                end
            end)
        elseif r < 27 then
            add_brush("pen", function()
                new_pen()
                if not got_brush then
                    music.change()
                    got_brush = true
                end
            end)
        elseif r < 36 then
            add_brush("lines", function()
                lines()
                if not got_brush then
                    music.change()
                    got_brush = true
                end
            end)
        elseif r < 40 then
            add_brush("rects", function()
                rects()
                if not got_brush then
                    music.change()
                    got_brush = true
                end
            end)
        elseif r < 70 then
            add_brush("color", function()
                new_color()
            end)
        elseif r < 78 then
            add_brush("bg", function()
                new_bg()
            end)
        elseif r < 88 then
            add_brush("fan", function()
                fan()
            end)
        else
            add_brush("scrape", function()
                scrape()
            end)
        end
    end

    make_new_collectables()

    main_layer:Action(function(dt)
        world:Step(dt)

        if not game_over then
            local y_force = 0
            local x_force = 0
            local vy = player_body.vy
            local vx = player_body.vx

            if is_jumping then
                if vy > 0 then
                    jump_force = jump_force - jump_force_decay * dt
                    y_force = jump_force
                    if jump_force < 0 then
                        jump_force = 0
                        is_jumping = false
                    end
                    --[[
                    if jump_time > max_jump_time then
                        is_jumping = false
                        jump_time = 0
                    else
                        jump_time = jump_time + dt
                    end
                    ]]
                else
                    is_jumping = false
                end
            end

            if run_dir == "left" then
                x_force = -run_force
            elseif run_dir == "right" then
                x_force = run_force
            else
                x_force = -vx * 0.0035
            end

            player_body:Force(x_force, y_force)
            local vx = player_body.vx
            if vx > run_speed then
                player_body.vx = run_speed
            elseif vx < -run_speed then
                player_body.vx = -run_speed
            end

            if prejump_timer > 0 then
                for _, fixture in ipairs(player_fixture:Touching()) do
                    if fixture.platform then
                        jump()
                        prejump_timer = 0
                        break
                    end
                end
                prejump_timer = prejump_timer - dt
            elseif prejump_timer < 0 then
                prejump_timer = 0
            end

            for _, fixture in ipairs(player_fixture:Touching()) do
                if fixture.brush then
                    first_time = false
                    for _, p in ipairs(platforms) do
                        if p ~= fixture.brush_platform and p.has_brush then
                            p.has_brush = nil
                        end
                    end
                    new_platforms()
                    fixture.action()
                    fixture.brush_platform.has_brush = nil
                    local old_emission_rate = trail.emission_rate
                    make_new_collectables()
                    if brushes % 7 == 0 and got_brush then
                        music.change()
                    end
                    break
                end
            end

            trail.source_position_x = player_body.x
            trail.source_position_y = player_body.y + 10

            if 
              get_indicator("health") <= 0 or
              get_indicator("money") <= 0 or
              get_indicator("inspiration") <= 0
            then
                remove_platforms()
                remove_brushes()
            end
            if player_body.y < lt.bottom - 100 then
                end_game()
            end

            brush_color = {
                math.random() * 0.04 + brush_color[1] - 0.02,
                math.random() * 0.04 + brush_color[2] - 0.02,
                math.random() * 0.04 + brush_color[3] - 0.02,
                brush_color[4]}
            trail.start_color_red = brush_color[1]
            trail.start_color_green = brush_color[2]
            trail.start_color_blue = brush_color[3]
            trail.end_color_red = trail.start_color_red
            trail.end_color_green = trail.start_color_green
            trail.end_color_blue = trail.start_color_blue


        end
        trail:Draw(canvas)
    end)

    lt.root.child = main_layer
end

start()
